/*
 * colname= one of {I,i,F,R,W,G} just like in Movie. You may also append
 * a "C" to make clipped values show up Red (ie colname=IC is like colname=I
 * but with clipped values flagged).
 * If "colname" is 3 characters or longer, it is assumed to give the name
 * of a color-table file.
 *
 * The float arrays red, green, and blue will be filled up with a raster
 * color table that is nocol elements long, in elements 0 through nocol-1.
 * The arrays should be declared 256 long in any case, however, so that
 * user-defined color tables are guaranteed to fit. The values in the
 * arrays range between 0. (full off) and 1. (full on).
 *
 * keywords: raster color-table vplot movie example utility
 */
#include <stdio.h>
#include <strings.h>

static int     *cola;
static int      cola2[9];
static float    rd_color ();
static float    gr_color ();
static float    bl_color ();

#define NONE	0
#define HALF	128
#define FULL	255
#define NEARFULL 200

static int      cintensityp[9] =
{
 NONE, HALF, FULL, NONE, HALF, FULL, NONE, HALF, FULL
};
static int      cintensityn[9] =
{
 FULL, HALF, NONE, FULL, HALF, NONE, FULL, HALF, NONE
};
static int      cflag[9] =
{
 FULL, FULL, NONE, NONE, FULL, NONE, NONE, FULL, FULL
};
static int      cbi[9] =
{
 FULL, NONE, NONE, NONE, NONE, NONE, NONE, NONE, FULL
};
static int      csurf[9] =
{
 NONE, NONE, FULL, NONE, NONE, FULL, NONE, NEARFULL, FULL
};
static int      cgsi[9] =
{
 NONE, FULL, FULL, NONE, FULL, NONE, NONE, FULL, NONE
};

name_to_coltab (colname, nocol, red, green, blue)
    char           *colname;
    int             nocol;
    float          *red, *green, *blue;
{
FILE           *fdc;
int             ii;

    if (nocol < 2 || nocol > 256)
	err ("nocol=%d in name_to_coltab is illegal\n", nocol);

    if (strlen (colname) <= 2)
    {
	switch (*colname)
	{
	case 'i':
	    cola = cintensityn;
	    break;
	case 'F':
	    cola = cflag;
	    break;
	case 'R':
	    cola = cbi;
	    break;
	case 'W':
	    cola = csurf;
	    break;
	case 'G':
	    cola = cgsi;
	    break;
	case 'I':
	default:
	    cola = cintensityp;
	    break;
	}

	for (ii = 0; ii < 9; ii++)
	{
	    switch (cola[ii])
	    {
	    case FULL:
		cola2[ii] = nocol - 1;
		break;
	    case NEARFULL:
		cola2[ii] = 200 * nocol / 256;
		break;
	    case HALF:
		cola2[ii] = nocol / 2;
		break;
	    case NONE:
	    default:
		cola2[ii] = 0;
		break;
	    }
	}

	for (ii = 0; ii < nocol; ii++)
	{
	    red[ii] = rd_color (ii, nocol);
	    green[ii] = gr_color (ii, nocol);
	    blue[ii] = bl_color (ii, nocol);
	}
/*
 * If clipping flagged, change the 2 values at either end to red
 * (Why 2? Remember that Movie throws half the colors away, so
 * this is what we're used to!)
 */
	if (colname[0] != '\0' && colname[1] == 'C')
	{
	    red[0] = red[1] = red[nocol-2] = red[nocol-1] = 1.;
	    green[0] = green[1] = green[nocol-2] = green[nocol-1] = 0.;
	    blue[0] = blue[1] = blue[nocol-2] = blue[nocol-1] = 0.;
	}
    }
    else
    {
/*
 * Get the colors out of a file. Unset colors are left black.
 */
	fdc = fopen (colname, "r");
	if (fdc == NULL)
	    err ("can't open colfile %s\n", colname);

	for (ii = 0; ii < nocol; ii++)
	{
	    red[ii] = 0.;
	    green[ii] = 0.;
	    blue[ii] = 0.;
	}

	while (fscanf (fdc, "%d ", &ii) != EOF)
	{
	    if (ii >= 256)
		err ("Maximum allowed color in colfile is 255\n");
	    if (ii < 0)
		err ("Minimum allowed color in colfile is 0\n");
	    fscanf (fdc, "%f %f %f\n", red + ii, green + ii, blue + ii);
	    if (red[ii] < 0. || green[ii] < 0. || blue[ii] < 0.)
		err ("Minimum allowed rgb value in colfile is 0.\n");
	    if (red[ii] > 1. || green[ii] > 1. || blue[ii] > 1.)
		err ("Maximum allowed rgb value in colfile is 1.\n");
	}
    }
}

static float
rd_color (x, ncol)
    int             x;
    int             ncol;
{
int             ncolo2;
    ncolo2 = ncol / 2.;

    if (x < ncolo2)
	return (((ncolo2 - x) * cola2[0] / (float) ncolo2) + (x * cola2[1] / (float) ncolo2)) / (float) (ncol - 1);
    else
	return ((((ncol - 1) - x) * cola2[1] / (float) (ncolo2 - 1)) + ((x - ncolo2) * cola2[2] / (float) (ncolo2 - 1))) / (float) (ncol - 1);
}

static float
gr_color (x, ncol)
    int             x;
    int             ncol;
{
int             ncolo2;
    ncolo2 = ncol / 2.;

    if (x < ncolo2)
	return (((ncolo2 - x) * cola2[3] / (float) ncolo2) + (x * cola2[4] / (float) ncolo2)) / (float) (ncol - 1);
    else
	return ((((ncol - 1) - x) * cola2[4] / (float) (ncolo2 - 1)) + ((x - ncolo2) * cola2[5] / (float) (ncolo2 - 1))) / (float) (ncol - 1);
}

static float
bl_color (x, ncol)
    int             x;
    int             ncol;
{
int             ncolo2;
    ncolo2 = ncol / 2.;

    if (x < ncolo2)
	return (((ncolo2 - x) * cola2[6] / (float) ncolo2) + (x * cola2[7] / (float) ncolo2)) / (float) (ncol - 1);
    else
	return ((((ncol - 1) - x) * cola2[7] / (float) (ncolo2 - 1)) + ((x - ncolo2) * cola2[8] / (float) (ncolo2 - 1))) / (float) (ncol - 1);
}
